from __future__ import annotations

from typing import Optional

import milkstraw_client
from milkstraw_client import APIClient


class Model:
    def __init__(self, id: str, name: str, status: str, source_data: str):
        self.id = id
        self.name = name
        self.status = status
        self.source_data = source_data

    def __repr__(self) -> str:
        attributes = ", ".join(f"{key}='{value}'" for key, value in vars(self).items())
        return f"{self.__class__.__name__}({attributes})"

    @staticmethod
    def create(name: str, source_data: str, anonymize_personal_info: Optional[bool] = None) -> Model:
        url = f"{milkstraw_client.edge_service_url}/models/"
        json = {"name": name, "sourceDataId": source_data}
        if anonymize_personal_info is not None:
            json["anonymizePersonalInfo"] = anonymize_personal_info
        response = APIClient.request("post", url, json=json)
        return Model.__parse_dict(response)

    @staticmethod
    def get(id: str) -> Model:
        url = f"{milkstraw_client.edge_service_url}/models/{id}"
        response = APIClient.request("get", url)
        return Model.__parse_dict(response)

    @staticmethod
    def list() -> list[Model]:
        url = f"{milkstraw_client.edge_service_url}/models"
        response = APIClient.request("get", url)
        models = [Model.__parse_dict(model_dict) for model_dict in response]
        return models

    @staticmethod
    def __parse_dict(model_dict: dict[str, str]) -> Model:
        return Model(
            id=model_dict["id"],
            name=model_dict["name"],
            status=model_dict["status"],
            source_data=model_dict["sourceData"],
        )



from __future__ import annotations

from typing import Union

import requests
from requests.exceptions import HTTPError

import milkstraw_client


class APIClient:
    token = None

    @classmethod
    def download_file(cls, file_path: str, url: str) -> str:
        response = APIClient.request("get", url, return_raw=True)
        with open(file_path, "wb") as f:
            f.write(response)
        return file_path

    @classmethod
    def request(
        cls,
        method: str,
        url: str,
        params: dict = {},
        json: dict = {},
        file_paths: dict[str, str] = {},
        return_raw=False,
    ) -> Union[dict, str]:
        cls.set_token()
        files = {file_param: open(file_path, "rb") for file_param, file_path in file_paths.items()}
        headers = {"Authorization": f"Bearer {cls.token}"}
        response = requests.request(method, url, headers=headers, params=params, json=json, files=files)
        if response.status_code not in {200, 201}:
            response_formatted = f"status_code: `{response.status_code}`, message: `{response.text}`"
            raise HTTPError(f"Request to Milkstraw API failed. {response_formatted}")
        if return_raw:
            return response.content
        json_response: dict = response.json()
        return json_response

    @classmethod
    def set_token(cls):
        if cls.token is not None:
            return
        login_data = {"email": milkstraw_client.user_email, "password": milkstraw_client.user_password}
        login_url = f"{milkstraw_client.edge_service_url}/users/login"
        response = requests.post(login_url, json=login_data)
        if response.status_code != 200:
            response_formatted = f"status_code: `{response.status_code}`, message: `{response.text}`"
            raise HTTPError(f"Failed to login to Milkstraw API. {response_formatted}")
        json_response: dict = response.json()
        cls.token = json_response.get("token")



from __future__ import annotations

import milkstraw_client
from milkstraw_client import APIClient


class SourceData:
    def __init__(self, id: str, name: str, status: str):
        self.id = id
        self.name = name
        self.status = status

    def __repr__(self) -> str:
        attributes = ", ".join(f"{key}='{value}'" for key, value in vars(self).items())
        return f"{self.__class__.__name__}({attributes})"

    @staticmethod
    def upload(name: str, file_path: str) -> SourceData:
        url = f"{milkstraw_client.edge_service_url}/source-data/"
        params = {"name": name}
        file_paths = {"file": file_path}
        response = APIClient.request("post", url, params=params, file_paths=file_paths)
        return SourceData(**response)

    @staticmethod
    def get(id: str) -> SourceData:
        url = f"{milkstraw_client.edge_service_url}/source-data/{id}"
        response = APIClient.request("get", url)
        return SourceData(**response)

    @staticmethod
    def list() -> list[SourceData]:
        url = f"{milkstraw_client.edge_service_url}/source-data"
        response = APIClient.request("get", url)
        data = [SourceData(**data_dict) for data_dict in response]
        return data

    @staticmethod
    def download(id: str, file_path: str) -> str:
        url = f"{milkstraw_client.edge_service_url}/source-data/download/{id}"
        return APIClient.download_file(file_path, url)



from __future__ import annotations

from typing import Optional

import milkstraw_client
from milkstraw_client import APIClient


class GeneratedData:
    def __init__(self, id: str, model: str, status: str):
        self.id = id
        self.model = model
        self.status = status

    def __repr__(self) -> str:
        attributes = ", ".join(f"{key}='{value}'" for key, value in vars(self).items())
        return f"{self.__class__.__name__}({attributes})"

    @staticmethod
    def generate(model: str, records_num: int, condition: Optional[dict] = None) -> GeneratedData:
        url = f"{milkstraw_client.edge_service_url}/generated-data/"
        json = {"modelId": model, "recordsNum": records_num}
        if condition is not None:
            json["condition"] = condition
        response = APIClient.request("post", url, json=json)
        return GeneratedData(**response)

    @staticmethod
    def get(id: str) -> GeneratedData:
        url = f"{milkstraw_client.edge_service_url}/generated-data/{id}"
        response = APIClient.request("get", url)
        return GeneratedData(**response)

    @staticmethod
    def list() -> list[GeneratedData]:
        url = f"{milkstraw_client.edge_service_url}/generated-data"
        response = APIClient.request("get", url)
        data = [GeneratedData(**data_dict) for data_dict in response]
        return data

    @staticmethod
    def download(id: str, file_path: str) -> str:
        url = f"{milkstraw_client.edge_service_url}/generated-data/download/{id}"
        return APIClient.download_file(file_path, url)

    @staticmethod
    def download_report(id: str, file_path: str) -> str:
        url = f"{milkstraw_client.edge_service_url}/generated-data/download/report/{id}"
        return APIClient.download_file(file_path, url)
